﻿using System.ComponentModel;
using System.Xml.Linq;

namespace SGLF.Core.Extensions
{
    public static class XmlExtensions
    {
        public static string Value(this XElement xelement, string defaultValue = "")
        {
            if (xelement != null)
                return xelement.Value;
            else
                return defaultValue;
        }

        public static T Value<T>(this XElement xelement, T defaultValue = default(T))
        {
            if (xelement != null)
            {
                try
                {
                    return (T) TypeDescriptor.GetConverter(typeof(T)).ConvertFromString(xelement.Value);
                }
                catch
                {
                    return defaultValue;
                }
            }
            else
                return defaultValue;
        }

        public static string Value(this XAttribute xattribute, string defaultValue = "")
        {
            if (xattribute != null)
                return xattribute.Value;
            else
                return defaultValue;
        }

        public static T Value<T>(this XAttribute xattribute, T defaultValue = default(T))
        {
            if (xattribute != null)
            {
                try
                {
                    return (T) TypeDescriptor.GetConverter(typeof(T)).ConvertFromString(xattribute.Value);
                }
                catch
                {
                    return defaultValue;
                }
            }
            else
                return defaultValue;
        }
    }
}