﻿using System;
using OpenTK;
using OpenTK.Graphics;

namespace SGLF.Core.Helpers
{
    public static class Vector3Helper
    {
        public static Vector3 FromColor(Color4 color)
        {
            return new Vector3(color.R, color.G, color.B);
        }

        public static Vector3 FromHSV(float h, float s, float v)
        {
            return FromHSV(new Vector3(h, s, v));
        }

        public static Vector3 FromHSV(Vector3 HSV)
        {
            float min;
            float chroma;
            float hdash;
            float x;
            Vector3 rgb = Vector3.Zero;

            chroma = HSV.Y * HSV.Z;
            hdash = (HSV.X % 360) / 60.0f;
            x = chroma * (1.0f - Math.Abs((hdash % 2.0f) - 1.0f));

            if (hdash < 1.0f)
            {
                rgb.X = chroma;
                rgb.Y = x;
            }
            else if (hdash < 2.0f)
            {
                rgb.X = x;
                rgb.Y = chroma;
            }
            else if (hdash < 3.0f)
            {
                rgb.Y = chroma;
                rgb.Z = x;
            }
            else if (hdash < 4.0f)
            {
                rgb.Y = x;
                rgb.Z = chroma;
            }
            else if (hdash < 5.0f)
            {
                rgb.X = x;
                rgb.Z = chroma;
            }
            else if (hdash <= 6.0f)
            {
                rgb.X = chroma;
                rgb.Z = x;
            }

            min = HSV.Z - chroma;

            rgb.X += min;
            rgb.Y += min;
            rgb.Z += min;

            return rgb;
        }

        public static Vector3 SphericalRandom(float radius, Random random)
        {
            Vector3 d = new Vector3
            (
                (float) random.NextDouble(),
                (float) random.NextDouble(),
                (float) random.NextDouble()
            );

            return d.Normalized() * radius;
        }

        public static Vector3 SphericalCoordinates(float phi, float theta, float radius)
        {
            Vector3 d = new Vector3
            (
                (float) (Math.Cos(phi) * Math.Sin(theta)),
                (float) (Math.Cos(theta)),
                (float) (Math.Sin(phi) * Math.Sin(theta))
            );

            return radius * d;
        }
    }
}