﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using OpenTK;

namespace SGLF.Core.Maths
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public struct BoundingBox
    {
        // ==================================================================================================
        // Members
        // ==================================================================================================

        public Vector3 Min;
        public Vector3 Max;

        // ==================================================================================================
        // Properties
        // ==================================================================================================

        public Vector3 Center
        {
            get
            {
                return (Min + Max) * 0.5f;
            }
        }

        public float Left
        {
            get
            {
                return Min.X;
            }
        }

        public float Right
        {
            get
            {
                return Max.X;
            }
        }

        public float Bottom
        {
            get
            {
                return Min.Y;
            }
        }

        public float Top
        {
            get
            {
                return Max.Y;
            }
        }

        public float Front
        {
            get
            {
                return Min.Z;
            }
        }

        public float Back
        {
            get
            {
                return Max.Z;
            }
        }

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        public BoundingBox(Vector3 point)
        {
            Min = Max = point;
        }

        public BoundingBox(Vector3 pointA, Vector3 pointB)
        {
            Min = new Vector3(Math.Min(pointA.X, pointB.X), Math.Min(pointA.Y, pointB.Y), Math.Min(pointA.Z, pointB.Z));
            Max = new Vector3(Math.Max(pointA.X, pointB.X), Math.Max(pointA.Y, pointB.Y), Math.Max(pointA.Z, pointB.Z));
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public bool TestPoint(Vector3 point)
        {
            return (point.X >= Min.X) && (point.Y >= Min.Y) && (point.Z >= Min.Z) &&
                   (point.X <= Max.X) && (point.Y <= Max.Y) && (point.Z <= Max.Z);
        }

        public bool TestIntersection(BoundingBox other)
        {
            return !(other.Left > Right || other.Right < Left || other.Bottom < Top || other.Top > Bottom ||
                     other.Front > Back || other.Back < Front);
        }
    }
}