﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using OpenTK;

namespace SGLF.Core.Maths
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public class Frustrum
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        public Plane[] Planes
        {
            get;
            private set;
        }

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        public Frustrum()
        {
            Planes = new Plane[6];
        }

        public Frustrum(Vector4 left, Vector4 right, Vector4 bottom, Vector4 top, Vector4 far, Vector4 near)
        {
            Planes = new Plane[6];
            Planes[0] = new Plane(left);
            Planes[1] = new Plane(right);
            Planes[2] = new Plane(bottom);
            Planes[3] = new Plane(top);
            Planes[4] = new Plane(far);
            Planes[5] = new Plane(near);
        }

        public Frustrum(Matrix4 projection)
        {
            Planes = new Plane[6];
            ExtractPlanes(projection);
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public void ExtractPlanes(Matrix4 projection)
        {
            // Left, Right
            Planes[0] = new Plane(projection.M14 + projection.M11, projection.M24 + projection.M21,
                                  projection.M34 + projection.M31, projection.M44 + projection.M41);
            Planes[1] = new Plane(projection.M14 - projection.M11, projection.M24 - projection.M21,
                                  projection.M34 - projection.M31, projection.M44 - projection.M41);
            // Bottom, Top
            Planes[2] = new Plane(projection.M14 + projection.M12, projection.M24 + projection.M22,
                                  projection.M34 + projection.M32, projection.M44 + projection.M42);
            Planes[3] = new Plane(projection.M14 - projection.M12, projection.M24 - projection.M22,
                                  projection.M34 - projection.M32, projection.M44 - projection.M42);
            // Near, Far
            Planes[4] = new Plane(projection.M14 + projection.M13, projection.M24 + projection.M23,
                                  projection.M34 + projection.M33, projection.M44 + projection.M43);
            Planes[5] = new Plane(projection.M14 - projection.M13, projection.M24 - projection.M23,
                                  projection.M34 - projection.M33, projection.M44 - projection.M43);
        }

        public bool TestPoint(Vector3 point)
        {
            for (int i = 0; i < 6; i++)
                if (!Planes[i].PointInPlane(point))
                    return false;

            return true;
        }

        public bool TestSphere(BoundingSphere sphere)
        {
            for (int i = 0; i < 6; i++)
                if (Planes[i].DistanceToPlane(sphere.Position) <= -sphere.Radius)
                    return false;

            return true;
        }

        public bool TestBox(BoundingBox aabb)
        {
            for (int i = 0; i < 6; i++)
            {
                if (Planes[i].DistanceToPlane(aabb.Min.X, aabb.Min.Y, aabb.Min.Z) > 0)
                    continue;
                if (Planes[i].DistanceToPlane(aabb.Min.X, aabb.Min.Y, aabb.Max.Z) > 0)
                    continue;
                if (Planes[i].DistanceToPlane(aabb.Min.X, aabb.Max.Y, aabb.Min.Z) > 0)
                    continue;
                if (Planes[i].DistanceToPlane(aabb.Min.X, aabb.Max.Y, aabb.Max.Z) > 0)
                    continue;
                if (Planes[i].DistanceToPlane(aabb.Max.X, aabb.Min.Y, aabb.Min.Z) > 0)
                    continue;
                if (Planes[i].DistanceToPlane(aabb.Max.X, aabb.Min.Y, aabb.Max.Z) > 0)
                    continue;
                if (Planes[i].DistanceToPlane(aabb.Max.X, aabb.Max.Y, aabb.Min.Z) > 0)
                    continue;
                if (Planes[i].DistanceToPlane(aabb.Max.X, aabb.Max.Y, aabb.Max.Z) > 0)
                    continue;
                return false;
            }
            return true;
        }
    }
}