﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using OpenTK;

namespace SGLF.Core.Maths
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public struct Plane
    {
        // ==================================================================================================
        // Members
        // ==================================================================================================

        public Vector3 Normal;

        public float Distance;

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        public Plane(float x, float y, float z, float w)
        {
            Normal = new Vector3(x, y, z);
            Distance = w;
        }

        public Plane(Vector3 normal, float distance)
        {
            Normal = normal;
            Distance = distance;
        }

        public Plane(Vector3 normal, Vector3 pointInPlane)
        {
            Normal = normal;
            Distance = -Vector3.Dot(pointInPlane, normal);
        }

        public Plane(Vector4 plane)
        {
            Normal = plane.Xyz;
            Distance = plane.W;
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public bool PointInPlane(float x, float y, float z)
        {
            return DistanceToPlane(x, y, z) <= 0;
        }

        public bool PointInPlane(Vector3 point)
        {
            return DistanceToPlane(point) <= 0;
        }

        public float DistanceToPlane(Vector3 point)
        {
            return Vector3.Dot(point, Normal) + Distance;
        }

        public float DistanceToPlane(float x, float y, float z)
        {
            return x * Normal.X + y * Normal.Y + z * Normal.Z + Distance;
        }

        public Vector3 ProjectOntoPlane(float x, float y, float z)
        {
            return ProjectOntoPlane(new Vector3(x, y, z));
        }

        public Vector3 ProjectOntoPlane(Vector3 point)
        {
            return point + Normal * DistanceToPlane(point);
        }
    }
}