﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using OpenTK;

namespace SGLF.Core.Maths
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public class Transform
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        public Transform Parent
        {
            get;
            set;
        }

        public Vector3 LocalPosition
        {
            get;
            set;
        }

        public Quaternion LocalRotation
        {
            get;
            set;
        }

        public Matrix4 Matrix
        {
            get
            {
                Matrix4 matrix = Matrix4.CreateFromQuaternion(LocalRotation) *
                                 Matrix4.CreateTranslation(LocalPosition);

                if (Parent != null)
                    matrix = Parent.Matrix * matrix;

                return matrix;
            }
        }

        public Matrix4 InvMatrix
        {
            get
            {
                return Matrix4.Invert(Matrix);
            }
        }

        public Vector3 Position
        {
            get
            {
                return Vector3.TransformPosition(Vector3.Zero, Matrix);
            }

            set
            {
                Matrix4 invMatrix = (Parent != null) ? Parent.InvMatrix : Matrix4.Identity;
                LocalPosition = Vector3.TransformPosition(value, invMatrix);
            }
        }

        public Quaternion Rotation
        {
            get
            {
                Quaternion rotation = LocalRotation;

                if (Parent != null)
                    rotation *= Parent.Rotation;

                return rotation;
            }

            set
            {
                LocalRotation = value;

                if (Parent != null)
                    LocalRotation *= Quaternion.Invert(Parent.Rotation);
            }
        }

        public Vector3 Right
        {
            get
            {
                return Vector3.Transform(new Vector3(1, 0, 0), Rotation);
            }
        }

        public Vector3 Up
        {
            get
            {
                return Vector3.Transform(new Vector3(0, 1, 0), Rotation);
            }
        }

        public Vector3 Forward
        {
            get
            {
                return Vector3.Transform(new Vector3(0, 0, -1), Rotation);
            }
        }

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        public Transform(Transform parent = null)
        {
            Parent = parent;
            LocalPosition = Vector3.Zero;
            LocalRotation = Quaternion.Identity;
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public void LookAt(Vector3 target, Vector3 up)
        {
            Matrix3 lookAtMatrix = new Matrix3(Matrix4.LookAt(Position, target, up));
            Rotation = Quaternion.FromMatrix(lookAtMatrix);
        }

        public void LookAt(Vector3 origin, Vector3 target, Vector3 up)
        {
            Position = origin;
            LookAt(target, up);
        }
    }
}