﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;

namespace SGLF.Core.Resources
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Represents a generic resource, implementing the necessary mechanims for their disposal.
    /// </summary>
    public abstract class Resource : IDisposable
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        /// <summary>
        /// A boolean indicating whether the resource has been disposed.
        /// </summary>
        public bool Disposed
        {
            get;
            private set;
        }

        // ==================================================================================================
        // Disposer
        // ==================================================================================================

        /// <summary>
        /// Finalizes an instance of the <see cref="Resource"/> class.
        /// </summary>
        ~Resource()
        {
            Dispose(false);
        }

        /// <summary>
        /// Disposes the instance's managed and unmanaged resources.
        /// </summary>
        public void Dispose()
        {
            Dispose(true);
        }

        /// <summary>
        /// Releases unmanaged and - optionally - managed resources.
        /// </summary>
        /// <param name="disposeManaged"><c>true</c> to release both managed and unmanaged resources;
        /// <c>false</c> to release only unmanaged resources.</param>
        public void Dispose(bool disposeManaged)
        {
            if (!Disposed)
            {
                Dispose(disposeManaged, AbleToDispose());
                Disposed = true;
            }
            GC.SuppressFinalize(this);
        }

        /// <summary>
        /// Determines whether this instance can dispose it's unmanaged resources.
        /// </summary>
        /// <returns><c>true</c> if possible, <c>false</c> otherwise.</returns>
        protected virtual bool AbleToDispose()
        {
            return true;
        }

        /// <summary>
        /// Disposes the instance's resources.
        /// </summary>
        /// <param name="disposeManaged"><c>true</c> to release managed resources.</param>
        /// <param name="disposeUnmanaged"><c>true</c> to release unmanaged resources.</param>
        protected virtual void Dispose(bool disposeManaged, bool disposeUnmanaged)
        {
        }
    }
}