﻿using System;
using OpenTK.Graphics.OpenGL;

namespace SGLF.Graphics.Buffers
{
    public sealed class IndexBuffer : AbstractBuffer
    {
        public PrimitiveType Primitive
        {
            get;
            private set;
        }

        public int IndexCount
        {
            get;
            private set;
        }

        public DrawElementsType IndexType
        {
            get;
            private set;
        }

        public IndexBuffer(PrimitiveType primitive, int indexCount, DrawElementsType indexType,
                                 BufferUsageHint usageHint)
            : base(BufferTarget.ElementArrayBuffer, usageHint, sizeof(int) * indexCount)
        {
            if (indexCount <= 0)
                throw new ArgumentOutOfRangeException("indexCount");

            Primitive = primitive;
            IndexCount = indexCount;
            IndexType = indexType;
        }

        /// <summary>
        /// Gets the contents of the buffer.
        /// </summary>
        /// <param name="index">Specifies the first index where the data read will begin.</param>
        /// <param name="data">Specifies the array that will store the buffer contents.</param>
        public void GetData(int index, int[] data)
        {
            // Check data / index
            if (data == null)
                throw new ArgumentNullException("data");
            if (index < 0 || index >= IndexCount)
                throw new ArgumentException("index");

            // Get data.
            GetRawData(sizeof(int) * index, data);
        }

        /// <summary>
        /// Sets the supplied data to the buffer.
        /// </summary>
        /// <param name="index">Specifies the first index where the data replacement will begin.</param>
        /// <param name="data">Specifies the array to the data that will be copied to the buffer.</param>
        public void SetData(int index, int[] data)
        {
            // Check data / index
            if (data == null)
                throw new ArgumentNullException("data");
            if (index < 0 || index >= IndexCount)
                throw new ArgumentException("index");

            // Set data.
            SetRawData(sizeof(int) * index, data);
        }
    }
}