﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Drawing;
using OpenTK;
using OpenTK.Graphics;
using SGLF.Core.Maths;

namespace SGLF.Graphics.Cameras
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public abstract class Camera
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        public Transform Transform
        {
            get;
            private set;
        }

        public Matrix4 ViewMatrix
        {
            get
            {
                return Transform.InvMatrix;
            }
        }

        public Vector2 ViewportCoords
        {
            get;
            set;
        }

        public Vector2 ViewportSize
        {
            get;
            set;
        }

        public float NearClipPlane
        {
            get;
            set;
        }

        public float FarClipPlane
        {
            get;
            set;
        }

        public Color4 ClearColor
        {
            get;
            set;
        }

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        public Camera(float nearClipPlane, float farClipPlane, Vector2 viewportCoords, Vector2 viewportSize)
        {
            Transform = new Transform();
            NearClipPlane = nearClipPlane;
            FarClipPlane = farClipPlane;
            ViewportCoords = viewportCoords;
            ViewportSize = viewportSize;
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public Rectangle GetViewportRectangle(int screenWidth, int screenHeight)
        {
            return new Rectangle
            (
                (int) Math.Ceiling(screenWidth * ViewportCoords.X),
                (int) Math.Ceiling(screenHeight * ViewportCoords.Y),
                (int) Math.Ceiling(screenWidth * ViewportSize.X),
                (int) Math.Ceiling(screenHeight * ViewportSize.Y)
            );
        }

        public Matrix3 GetViewportMatrix()
        {
            return new Matrix3
            (
                ViewportSize.X, 0, ViewportCoords.X,
                0, ViewportSize.Y, ViewportCoords.Y,
                0, 0, 1
            );
        }

        public abstract Matrix4 GetProjectionMatrix(float aspectRatio);
    }
}