﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using OpenTK.Graphics.OpenGL;

namespace SGLF.Graphics
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Static Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Stores the capabilities and limits of the different resources of the graphics device.
    /// </summary>
    public static class GraphicsCapabilities
    {
        // ==================================================================================================
        // Members
        // ==================================================================================================

        private static int maxRenderbufferSize;
        private static int maxCombinedTextureImageUnits;
        private static int maxTextureSize;
        private static int maxArrayTextureLayers;
        private static int maxTexture3DSize;
        private static int maxCubeMapTextureSize;
        private static int maxTextureBufferSize;
        private static float maxTextureAnisotropy;
        private static int maxDrawBuffers;
        private static int maxVertexAttribs;

        // ==================================================================================================
        // Properties
        // ==================================================================================================

        /// <summary>
        /// The maximum size of a render buffer.
        /// </summary>
        public static int MaxRenderbufferSize
        {
            get
            {
                return maxRenderbufferSize;
            }
        }

        /// <summary>
        /// The maximum number of texture units.
        /// </summary>
        public static int MaxCombinedTextureImageUnits
        {
            get
            {
                return maxCombinedTextureImageUnits;
            }
        }

        /// <summary>
        /// The maximum size for a texture.
        /// </summary>
        public static int MaxTextureSize
        {
            get
            {
                return maxTextureSize;
            }
        }

        /// <summary>
        /// The maximum number of layers for a texture array.
        /// </summary>
        public static int MaxArrayTextureLayers
        {
            get
            {
                return maxArrayTextureLayers;
            }
        }

        /// <summary>
        /// The maximum size for a 3D texture.
        /// </summary>
        public static int MaxTexture3DSize
        {
            get
            {
                return maxTexture3DSize;
            }
        }

        /// <summary>
        /// The maximum size for a cube map texture.
        /// </summary>
        public static int MaxCubeMapTextureSize
        {
            get
            {
                return maxCubeMapTextureSize;
            }
        }

        /// <summary>
        /// The maximum size for a texture buffer.
        /// </summary>
        public static int MaxTextureBufferSize
        {
            get
            {
                return maxTextureBufferSize;
            }
        }

        /// <summary>
        /// The maximum texture anisotropy.
        /// </summary>
        public static float MaxTextureAnisotropy
        {
            get
            {
                return maxTextureAnisotropy;
            }
        }

        /// <summary>
        /// The maximum number of draw buffers.
        /// </summary>
        public static int MaxDrawBuffers
        {
            get
            {
                return maxDrawBuffers;
            }
        }

        /// <summary>
        /// The maximum number of vertex attributes.
        /// </summary>
        public static int MaxVertexAttribs
        {
            get
            {
                return maxVertexAttribs;
            }
        }

        // ==================================================================================================
        // Static Constructor
        // ==================================================================================================

        /// <summary>
        /// Initializes the <see cref="GraphicsCapabilities"/> class.
        /// </summary>
        static GraphicsCapabilities()
        {
            GL.GetInteger(GetPName.MaxRenderbufferSize, out maxRenderbufferSize);
            GL.GetInteger(GetPName.MaxCombinedTextureImageUnits, out maxCombinedTextureImageUnits);
            GL.GetInteger(GetPName.MaxTextureSize, out maxTextureSize);
            GL.GetInteger(GetPName.MaxArrayTextureLayers, out maxArrayTextureLayers);
            GL.GetInteger(GetPName.Max3DTextureSize, out maxTexture3DSize);
            GL.GetInteger(GetPName.MaxCubeMapTextureSize, out maxCubeMapTextureSize);
            GL.GetInteger(GetPName.MaxTextureBufferSize, out maxTextureBufferSize);
            GL.GetFloat((GetPName) All.MaxTextureMaxAnisotropyExt, out maxTextureAnisotropy);
            GL.GetInteger(GetPName.MaxDrawBuffers, out maxDrawBuffers);
            GL.GetInteger(GetPName.MaxVertexAttribs, out maxVertexAttribs);
        }
    }
}