﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System.Collections.Generic;
using System.Linq;
using SGLF.Graphics.Managers;

namespace SGLF.Graphics
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Static Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// A global Graphics resource manager.
    /// </summary>
    public static class GraphicsManager
    {
        // ==================================================================================================
        // Static Members
        // ==================================================================================================

        private static HashSet<GraphicsResource> Resources;

        // ==================================================================================================
        // Static Properties
        // ==================================================================================================

        /// <summary>
        /// The Material manager.
        /// </summary>
        public static MaterialManager Materials
        {
            get;
            private set;
        }

        /// <summary>
        /// The Model manager.
        /// </summary>
        public static ModelManager Models
        {
            get;
            private set;
        }

        /// <summary>
        /// The Shader manager.
        /// </summary>
        public static ProgramManager Programs
        {
            get;
            private set;
        }

        /// <summary>
        /// The Texture manager.
        /// </summary>
        public static TextureManager Textures
        {
            get;
            private set;
        }

        /// <summary>
        /// The Framebuffer manager.
        /// </summary>
        public static ColorbufferPool Colorbuffer
        {
            get;
            private set;
        }

        // ==================================================================================================
        // Static Constructor
        // ==================================================================================================

        /// <summary>
        /// Initializes the <see cref="GraphicsManager"/> class.
        /// </summary>
        static GraphicsManager()
        {
            // Initialize managers.
            Resources = new HashSet<GraphicsResource>();
            Materials = new MaterialManager();
            Models = new ModelManager();
            Programs = new ProgramManager();
            Textures = new TextureManager();
            Colorbuffer = new ColorbufferPool();

            // Initialize default manager paths.
            Materials.BasePath = "Resources/Materials/";
            Models.BasePath = "Resources/Models/";
            Programs.BasePath = "Resources/Shaders/";
            Textures.BasePath = "Resources/Textures/";
        }

        // ==================================================================================================
        // Static Methods
        // ==================================================================================================

        /// <summary>
        /// Registers the specified resource.
        /// </summary>
        /// <returns><c>true</c> if successful, <c>false</c> otherwise.</returns>
        public static bool Register(GraphicsResource T)
        {
            return Resources.Add(T);
        }

        /// <summary>
        /// Unregisters the specified resource.
        /// </summary>
        /// <returns><c>true</c> if successful, <c>false</c> otherwise.</returns>
        public static bool Unregister(GraphicsResource T)
        {
            return Resources.Remove(T);
        }

        /// <summary>
        /// Disposes all the resources.
        /// </summary>
        public static void Dispose()
        {
            // Dispose managers.
            Materials.Dispose();
            Models.Dispose();
            Programs.Dispose();
            Textures.Dispose();

            // Dispose pending registered.
            foreach (var resource in Resources.ToArray())
                resource.Dispose(true);
        }
    }
}