﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;

namespace SGLF.Graphics
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Represents an OpenGL resource, containing the object's named handle and implementing the necessary
    /// mechanims for their disposal.
    /// </summary>
    public abstract class GraphicsNamedResource : GraphicsResource
    {
        // ==================================================================================================
        // Members
        // ==================================================================================================

        private int handle;

        // ==================================================================================================
        // Properties
        // ==================================================================================================

        /// <summary>
        /// Retrieves this object's named handle, used by OpenGL to identify the different resources. In case
        /// the GraphicsContext has been disposed, it'll throw an <see cref="ObjectDisposeException"/>.
        /// </summary>
        public int Handle
        {
            get
            {
                if (Disposed)
                    throw new ObjectDisposedException(GetType().Name);

                return handle;
            }

            protected set
            {
                handle = value;
            }
        }

        // ==================================================================================================
        // Constructor / Disposer
        // ==================================================================================================

        /// <summary>
        /// Determines whether this instance can dispose it's unmanaged resources.
        /// </summary>
        /// <returns><c>true</c> if possible, <c>false</c> otherwise.</returns>
        protected override bool AbleToDispose()
        {
            return GraphicsState.Context.IsCurrent && !GraphicsState.Context.IsDisposed;
        }
    }
}