﻿using System.Collections.Generic;
using OpenTK.Graphics.OpenGL;
using SGLF.Graphics.Textures;

namespace SGLF.Graphics.Managers
{
    public class ColorbufferPool
    {
        private const int DefaultLifeTime = 5;

        private class PooledInstance
        {
            public int Width;
            public int Height;
            public int LifeTime;
            public InternalFormat Format;
            public Colorbuffer Colorbuffer;
        }

        private List<PooledInstance> Instances;

        public ColorbufferPool()
        {
            Instances = new List<PooledInstance>();
        }

        public Colorbuffer Acquire(int width, int height, InternalFormat format)
        {
            int index = Find(width, height, format);

            if (index != -1)
            {
                Colorbuffer colorbuffer = Instances[index].Colorbuffer;
                Instances.RemoveAt(index);
                return colorbuffer;
            }
            else
                return new Colorbuffer(width, height, format);
        }

        public void Release(Colorbuffer colorbuffer)
        {
            PooledInstance instance = new PooledInstance();
            instance.Width = colorbuffer.Width;
            instance.Height = colorbuffer.Height;
            instance.Format = colorbuffer.Color.Format;
            instance.Colorbuffer = colorbuffer;
            instance.LifeTime = DefaultLifeTime;

            Instances.Add(instance);
        }

        public void Recollect()
        {
            for (int i = 0; i < Instances.Count; i++)
            {
                if (--Instances[i].LifeTime == 0)
                    Instances[i].Colorbuffer.Dispose();
            }

            Instances.RemoveAll(i => i.LifeTime == 0);
        }

        private int Find(int width, int height, InternalFormat format)
        {
            for (int i = 0; i < Instances.Count; i++)
            {
                if (Instances[i].Width == width && Instances[i].Height == height && Instances[i].Format == format)
                    return i;
            }

            return -1;
        }
    }
}