﻿using System.Xml.Linq;
using OpenTK;
using OpenTK.Graphics.OpenGL;
using SGLF.Core.Extensions;
using SGLF.Core.Helpers;
using SGLF.Core.Resources;
using SGLF.Graphics.Materials;
using SGLF.Graphics.Shaders;
using SGLF.Graphics.Textures;

namespace SGLF.Graphics.Managers
{
    public class MaterialManager : ResourceManager<Material>
    {
        protected override Material LoadFromFile(string fileName)
        {
            XElement root = XElement.Load(fileName);
            return LoadMaterial(root);
        }

        public Material LoadMaterial(XElement root)
        {
            Material material = new Material();

            LoadProgram(root.Element("program"), material);
            LoadUniforms(root.Element("uniforms"), material);

            return material;
        }

        private void LoadProgram(XElement subProgram, Material material)
        {
            string name = subProgram.Attribute("filename").Value();
            material.Program = GraphicsManager.Programs.Load(name);
        }

        private void LoadUniforms(XElement subUniforms, Material material)
        {
            foreach (var subUniform in subUniforms.Elements("uniform"))
            {
                Uniform uniform = null;
                string type = subUniform.Attribute("type").Value();
                string name = subUniform.Attribute("name").Value();

                switch (type)
                {
                    case "int":
                        uniform = new Uniform<int>(name, subUniform.Attribute("value").Value<int>());
                        break;

                    case "float":
                        uniform = new Uniform<float>(name, subUniform.Attribute("value").Value<float>());
                        break;

                    case "vector2":
                        uniform = new Uniform<Vector2>(name, XmlHelper.GetVector2(subUniform));
                        break;

                    case "vector3":
                        uniform = new Uniform<Vector3>(name, XmlHelper.GetVector3(subUniform));
                        break;

                    case "vector4":
                        uniform = new Uniform<Vector4>(name, XmlHelper.GetVector4(subUniform));
                        break;

                    case "sampler":
                        TextureObject texture = GraphicsManager.Textures.Load(subUniform.Attribute("filename").Value());
                        TextureUnit textureUnit = TextureUnit.Texture0 + subUniform.Attribute("unit").Value<int>();
                        uniform = new UniformSampler(name, texture, textureUnit);
                        break;
                }

                material.UniformSet.Add(uniform);
            }
        }
    }
}