﻿using OpenTK.Graphics.OpenGL;
using SGLF.Graphics.Buffers;
using SGLF.Graphics.Vertices;

namespace SGLF.Graphics.Models
{
    public class Mesh : GraphicsResource
    {
        public VertexFormat VertexFormat
        {
            get;
            private set;
        }

        public VertexBuffer VertexBuffer
        {
            get;
            private set;
        }

        public IndexBuffer IndexBuffer
        {
            get;
            private set;
        }

        public VertexArrayObject VertexArray
        {
            get;
            private set;
        }

        public float BoundingRadius
        {
            get;
            set;
        }

        public Mesh(VertexFormat vertexFormat, int vertexCount, PrimitiveType primitive,
                    int indexCount, DrawElementsType indexType, bool isDynamic)
        {
            VertexFormat = vertexFormat;

            BufferUsageHint usageHint = (isDynamic) ? BufferUsageHint.DynamicDraw :
                                                      BufferUsageHint.StaticDraw;

            IndexBuffer = new IndexBuffer(primitive, indexCount, indexType, usageHint);
            VertexBuffer = new VertexBuffer(vertexFormat, vertexCount, usageHint);

            VertexArray = new VertexArrayObject();
            VertexArray.Attach(VertexBuffer, VertexFormat.VertexAttributes);
        }
    }
}