﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using OpenTK.Graphics.OpenGL;
using SGLF.Graphics.Buffers;
using SGLF.Graphics.Cameras;
using SGLF.Graphics.Materials;
using SGLF.Graphics.Models;
using SGLF.Graphics.Shaders;
using SGLF.Graphics.Vertices;

namespace SGLF.Graphics
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public sealed class RendererForward : Renderer
    {
        // ==================================================================================================
        // Constructor / Disposer
        // ==================================================================================================

        public RendererForward()
        {
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public override void Render()
        {
            GL.Enable(EnableCap.CullFace);
            GL.CullFace(CullFaceMode.Back);
            GL.DepthFunc(DepthFunction.Lequal);

            // Render each camera.
            foreach (Camera camera in Cameras)
            {
                // Bind camera.
                CameraBind(camera);
                SetScreenViewport();

                // Clear screen.
                GL.ClearColor(camera.ClearColor);
                GL.Clear(ClearBufferMask.ColorBufferBit | ClearBufferMask.DepthBufferBit);

                // Render.
                GeometryPass();
            }
        }

        private void SetScreenViewport()
        {
            GL.Viewport(Viewport.X, Viewport.Y, Viewport.Width, Viewport.Height);
        }

        private void GeometryPass()
        {
            // Setup blending.
            GL.BlendFunc(BlendingFactorSrc.One, BlendingFactorDest.One);

            // Per each entity...
            for (int i = 0; i < RenderEntities.Count; i++)
            {
                EntityBind(i);

                // Per each mesh...
                for (int j = 0; j < RenderEntities[i].Model.Meshes.Count; j++)
                {
                    Mesh mesh = RenderEntities[i].Model.Meshes[j];
                    Material material = RenderEntities[i].Model.Materials[j];
                    IndexBuffer indices = mesh.IndexBuffer;
                    VertexArrayObject vertexArray = mesh.VertexArray;

                    // Bind mesh.
                    GraphicsState.BindVertexArray(vertexArray);
                    GraphicsState.BindBuffer(indices);

                    // Bind material.
                    Program program = material.Program.Get();
                    GraphicsState.BindProgram(program);
                    program.SetUniform(UScene.UniformSet);
                    program.SetUniform("uMaterial.", material.UniformSet);

                    // Bind textures.
                    material.UniformSet.ForEachUniform<UniformSampler>(u =>
                        GraphicsState.BindTexture(u.TextureUnit, u.Texture));

                    // Disable blending.
                    GL.Disable(EnableCap.Blend);

                    // Per each light...
                    for (int k = 0; k < RenderLights.Count; k += MaxLightsPerBatch)
                    {
                        UNumLights.Value = Math.Min(RenderLights.Count - k, MaxLightsPerBatch);

                        for (int l = 0; l < UNumLights.Value; l++)
                            LightBind(k + l, l);

                        program.SetUniform(ULightUniformSet);

                        // Render mesh.
                        GL.DrawElements(indices.Primitive, indices.IndexCount, indices.IndexType, 0);
                        GL.Enable(EnableCap.Blend);
                    }
                }
            }
        }
    }
}