﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;

namespace SGLF.Graphics.Shaders
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Represents an uniform. They can be updated in two fashions: from their fixed location in the shader
    /// program or through their name.
    /// </summary>
    public abstract class Uniform
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        /// <summary>
        /// The location of the uniform.
        /// </summary>
        public int Location
        {
            get;
            private set;
        }

        /// <summary>
        /// The name of the uniform.
        /// </summary>
        public string Name
        {
            get;
            private set;
        }

        /// <summary>
        /// The value type of this uniform.
        /// </summary>
        public Type Type
        {
            get;
            private set;
        }

        /// <summary>
        /// Retrieves wether or not the uniform has a known binding location or it's binded by name.
        /// </summary>
        public bool HasLocation
        {
            get
            {
                return Location != -1;
            }
        }

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        /// <summary>
        /// Initializes a new instance of the <see cref="Uniform"/> class.
        /// </summary>
        /// <param name="name">The uniform's name.</param>
        /// <param name="type">The uniform's value type.</param>
        protected Uniform(string name, Type type)
        {
            Location = -1;
            Name = name;
            Type = type;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Uniform"/> class.
        /// </summary>
        /// <param name="location">The uniform's location.</param>
        /// <param name="type">The uniform's value type.</param>
        protected Uniform(int location, Type type)
        {
            Location = location;
            Name = "";
            Type = type;
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public abstract void SetUniform(Program program, string baseName = "");
    }
}