﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

namespace SGLF.Graphics.Shaders
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Represents a generic <see cref="Uniform"/>.
    /// </summary>
    public class Uniform<T> : Uniform
        where T : struct
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        /// <summary>
        /// The value of the uniform.
        /// </summary>
        public T Value
        {
            get;
            set;
        }

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        /// <summary>
        /// Initializes a new instance of the <see cref="Uniform"/> class.
        /// </summary>
        /// <param name="name">The uniform's name.</param>
        /// <param name="value">The uniform's value.</param>
        public Uniform(string name, T value = default(T))
            : base(name, typeof(T))
        {
            Value = value;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="Uniform"/> class.
        /// </summary>
        /// <param name="location">The uniform's location.</param>
        /// <param name="value">The uniform's value.</param>
        public Uniform(int location, T value = default(T))
            : base(location, typeof(T))
        {
            Value = value;
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public override void SetUniform(Program program, string baseName = "")
        {
            dynamic value = Value;

            if (HasLocation)
                program.SetUniform(Location, value);
            else
                program.SetUniform(baseName + Name, value);
        }
    }
}