﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using OpenTK.Graphics.OpenGL;
using SGLF.Graphics.Textures;

namespace SGLF.Graphics.Shaders
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Represents a sampler <see cref="Uniform"/>.
    /// </summary>
    public class UniformSampler : Uniform
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        public TextureObject Texture
        {
            get;
            set;
        }

        public TextureUnit TextureUnit
        {
            get;
            set;
        }

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        /// <summary>
        /// Initializes a new instance of the <see cref="UniformSampler"/> class.
        /// </summary>
        /// <param name="name">The uniform's name.</param>
        /// <param name="texture">The uniform's texture.</param>
        /// <param name="textureUnit">The uniform's texture unit.</param>
        public UniformSampler(string name, TextureObject texture = null, TextureUnit textureUnit = TextureUnit.Texture0)
            : base(name, typeof(TextureObject))
        {
            Texture = texture;
            TextureUnit = textureUnit;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="UniformSampler"/> class.
        /// </summary>
        /// <param name="location">The uniform's location.</param>
        /// <param name="texture">The uniform's texture.</param>
        /// <param name="textureUnit">The uniform's texture unit.</param>
        public UniformSampler(int location, TextureObject texture = null, TextureUnit textureUnit = TextureUnit.Texture0)
            : base(location, typeof(TextureObject))
        {
            Texture = texture;
            TextureUnit = textureUnit;
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public override void SetUniform(Program program, string baseName = "")
        {
            int textureUnit = (int) (TextureUnit - TextureUnit.Texture0);

            if (HasLocation)
                program.SetUniform(Location, textureUnit);
            else
                program.SetUniform(baseName + Name, textureUnit);
        }
    }
}