﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Linq;
using OpenTK.Graphics.OpenGL;
using SGLF.Graphics.Textures;

namespace SGLF.Graphics.Shaders
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public class UniformSet
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        public HashSet<Uniform> Uniforms
        {
            get;
            private set;
        }

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        public UniformSet()
        {
            Uniforms = new HashSet<Uniform>();
        }

        public UniformSet(IEnumerable<Uniform> uniforms)
        {
            Uniforms = new HashSet<Uniform>(uniforms);
        }

        public UniformSet(params Uniform[] uniforms)
        {
            Uniforms = new HashSet<Uniform>(uniforms);
        }

        // ==================================================================================================
        // Methods
        // ==================================================================================================

        public void Add(Uniform uniform)
        {
            Uniforms.Add(uniform);
        }

        public void ForEachUniform<T>(Action<T> action)
            where T : Uniform
        {
            foreach (T u in Uniforms.Select(u => u as T).Where(u => u != null))
                action(u);
        }

        public T GetValue<T>(string name)
            where T : struct
        {
            Uniform<T> uniform = (Uniform<T>) Uniforms.Where(u => (u.Name == name)).Single();

            if (uniform == null)
                throw new KeyNotFoundException();

            return uniform.Value;
        }

        public T GetValue<T>(int location)
            where T : struct
        {
            Uniform<T> uniform = (Uniform<T>) Uniforms.Where(u => (u.Location == location)).Single();

            if (uniform == null)
                throw new KeyNotFoundException();

            return uniform.Value;
        }

        public void GetSampler(string name, out TextureObject texture, out TextureUnit textureUnit)
        {
            UniformSampler uniform = (UniformSampler) Uniforms.Where(u => (u.Name == name)).Single();

            if (uniform == null)
                throw new KeyNotFoundException();

            texture = uniform.Texture;
            textureUnit = uniform.TextureUnit;
        }

        public void GetSampler(int location, out TextureObject texture, out TextureUnit textureUnit)
        {
            UniformSampler uniform = (UniformSampler) Uniforms.Where(u => (u.Location == location)).Single();

            if (uniform == null)
                throw new KeyNotFoundException();

            texture = uniform.Texture;
            textureUnit = uniform.TextureUnit;
        }

        public void SetValue<T>(string name, T value)
            where T : struct
        {
            Uniform<T> uniform = (Uniform<T>) Uniforms.Where(u => (u.Name == name)).Single();

            if (uniform == null)
                throw new KeyNotFoundException();

            uniform.Value = value;
        }

        public void SetSampler(string name, TextureObject texture, TextureUnit textureUnit)
        {
            UniformSampler uniform = (UniformSampler) Uniforms.Where(u => (u.Name == name)).Single();

            if (uniform == null)
                throw new KeyNotFoundException();

            uniform.Texture = texture;
            uniform.TextureUnit = textureUnit;
        }
    }
}