﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using OpenTK.Graphics.OpenGL;

namespace SGLF.Graphics.Textures
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public class Colorbuffer : GraphicsResource
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        public int Width
        {
            get
            {
                return Color.Width;
            }
        }

        public int Height
        {
            get
            {
                return Color.Height;
            }
        }

        public InternalFormat Format
        {
            get
            {
                return Color.Format;
            }
        }

        public Framebuffer Framebuffer
        {
            get;
            private set;
        }

        public Texture2D Color
        {
            get;
            private set;
        }

        // ==================================================================================================
        // Constructor / Disposer
        // ==================================================================================================

        public Colorbuffer(int width, int height, InternalFormat format)
        {
            // Generate texture.
            Color = new Texture2D(width, height, format, 0);
            Color.MagFilter = TextureMagFilter.Nearest;
            Color.MinFilter = TextureMinFilter.Nearest;
            Color.WrapModeS = TextureWrapMode.Clamp;
            Color.WrapModeT = TextureWrapMode.Clamp;

            // Generate framebuffer.
            Framebuffer = new Framebuffer(width, height);
            Framebuffer.Attach(FramebufferAttachment.ColorAttachment0, Color);
            Framebuffer.Realize();
        }

        /// <summary>
        /// Disposes the instance's resources.
        /// </summary>
        /// <param name="disposeManaged"><c>true</c> to release managed resources.</param>
        /// <param name="disposeUnmanaged"><c>true</c> to release unmanaged resources.</param>
        protected override void Dispose(bool disposeManaged, bool disposeUnmanaged)
        {
            if (disposeUnmanaged)
            {
                if (Framebuffer != null)
                    Framebuffer.Dispose();

                if (Color != null)
                    Color.Dispose();
            }

            base.Dispose(disposeManaged, disposeUnmanaged);
        }
    }
}