﻿using OpenTK.Graphics.OpenGL;

namespace SGLF.Graphics.Textures
{
    public abstract class TextureObject : GraphicsNamedResource
    {
        private TextureMagFilter magFilter;
        private TextureMinFilter minFilter;
        private TextureWrapMode wrapModeS;
        private TextureWrapMode wrapModeT;

        public InternalFormat Format
        {
            get;
            protected set;
        }

        public TextureMagFilter MagFilter
        {
            get
            {
                return magFilter;
            }
            set
            {
                magFilter = value;
                int temp = (int) value;
                GL.Ext.TextureParameterI(Handle, Target, TextureParameterName.TextureMagFilter, ref temp);
            }
        }

        public TextureMinFilter MinFilter
        {
            get
            {
                return minFilter;
            }
            set
            {
                minFilter = value;
                int temp = (int) value;
                GL.Ext.TextureParameterI(Handle, Target, TextureParameterName.TextureMinFilter, ref temp);
            }
        }

        public int NumMipMaps
        {
            get;
            private set;
        }

        public TextureTarget Target
        {
            get;
            private set;
        }

        public TextureWrapMode WrapModeS
        {
            get
            {
                return wrapModeS;
            }
            set
            {
                wrapModeS = value;
                int temp = (int) value;
                GL.Ext.TextureParameterI(Handle, Target, TextureParameterName.TextureWrapS, ref temp);
            }
        }

        public TextureWrapMode WrapModeT
        {
            get
            {
                return wrapModeT;
            }
            set
            {
                wrapModeT = value;
                int temp = (int) value;
                GL.Ext.TextureParameterI(Handle, Target, TextureParameterName.TextureWrapT, ref temp);
            }
        }

        protected TextureObject(InternalFormat format, TextureTarget target, int numMipMaps)
        {
            Handle = GL.GenTexture();
            Format = format;
            Target = target;
            NumMipMaps = numMipMaps;
            MinFilter = TextureMinFilter.Linear;
            MagFilter = TextureMagFilter.Linear;
            WrapModeS = TextureWrapMode.Repeat;
            WrapModeT = TextureWrapMode.Repeat;
        }

        /// <summary>
        /// Disposes the instance's resources.
        /// </summary>
        /// <param name="disposeManaged"><c>true</c> to release managed resources.</param>
        /// <param name="disposeUnmanaged"><c>true</c> to release unmanaged resources.</param>
        protected override void Dispose(bool disposeManaged, bool disposeUnmanaged)
        {
            if (disposeUnmanaged)
                GL.DeleteTexture(Handle);

            base.Dispose(disposeManaged, disposeUnmanaged);
        }
    }
}