﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using OpenTK;

namespace SGLF.Graphics.Vertices
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Interfaces
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Provides a common interface to Vertex structures, allowing access to the vertex format and
    /// common vertex elements.
    /// </summary>
    public interface IVertex
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        /// <summary>
        /// Gets the <see cref="VertexFormat"/> describing the Vertex's layout and data.
        /// </summary>
        VertexFormat Format
        {
            get;
        }

        /// <summary>
        /// The position vector.
        /// </summary>
        Vector3 Position
        {
            get;
            set;
        }

        /// <summary>
        /// The tangent vector.
        /// </summary>
        Vector4 Tangent
        {
            get;
            set;
        }

        /// <summary>
        /// The normal vector.
        /// </summary>
        Vector3 Normal
        {
            get;
            set;
        }

        /// <summary>
        /// The texture coordinates vector.
        /// </summary>
        Vector2 TexCoords
        {
            get;
            set;
        }

        /// <summary>
        /// The color vector.
        /// </summary>
        Vector4 Color
        {
            get;
            set;
        }
    }
}