﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using OpenTK.Graphics.OpenGL;
using SGLF.Graphics.Buffers;

namespace SGLF.Graphics.Vertices
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    ///
    /// </summary>
    public sealed class VertexArrayObject : GraphicsNamedResource
    {
        // ==================================================================================================
        // Sub-Structures
        // ==================================================================================================

        /// <summary>
        ///
        /// </summary>
        private struct AttributeState
        {
            public VertexAttribute attribute;
            public AbstractBuffer buffer;
        }

        // ==================================================================================================
        // Constants
        // ==================================================================================================

        private List<AttributeState> attributes = new List<AttributeState>();

        // ==================================================================================================
        // Constructor / Disposer
        // ==================================================================================================

        /// <summary>
        /// Initializes a new instance of the <see cref="VertexArrayObject"/> class.
        /// </summary>
        public VertexArrayObject()
        {
            Handle = GL.GenVertexArray();
        }

        /// <summary>
        /// Disposes the instance's resources.
        /// </summary>
        /// <param name="disposeManaged"><c>true</c> to release managed resources.</param>
        /// <param name="disposeUnmanaged"><c>true</c> to release unmanaged resources.</param>
        protected override void Dispose(bool disposeManaged, bool disposeUnmanaged)
        {
            if (disposeUnmanaged)
                GL.DeleteVertexArray(Handle);

            base.Dispose(disposeManaged, disposeUnmanaged);
        }

        // ==================================================================================================
        // Constructor / Disposer
        // ==================================================================================================

        /// <summary>
        /// Attaches the specified buffer.
        /// </summary>
        /// <param name="buffer">The buffer.</param>
        /// <param name="attribute">The attribute.</param>
        public void Attach(AbstractBuffer buffer, VertexAttribute attribute)
        {
            AttributeState state = new AttributeState();
            state.buffer = buffer;
            state.attribute = attribute;
            attributes.Add(state);

            GL.Ext.EnableVertexArrayAttrib(Handle, attribute.AttributeIndex);
            GL.Ext.VertexArrayVertexAttribOffset(Handle, buffer.Handle, attribute.AttributeIndex,
                                                 (int) attribute.Components, attribute.ComponentsType,
                                                 attribute.Normalized, (int) attribute.Stride,
                                                 (IntPtr) attribute.Offset);
        }

        /// <summary>
        /// Attaches the specified buffer.
        /// </summary>
        /// <param name="buffer">The buffer.</param>
        /// <param name="attributes">The attributes.</param>
        public void Attach(AbstractBuffer buffer, VertexAttribute[] attributes)
        {
            foreach (var attribute in attributes)
                Attach(buffer, attribute);
        }

        /// <summary>
        /// Clears this instance.
        /// </summary>
        public void Clear()
        {
            for (int i = 0; i < GraphicsCapabilities.MaxVertexAttribs; i++)
                GL.Ext.DisableVertexArrayAttrib(Handle, 0);

            attributes.Clear();
        }
    }
}