﻿// TODO: Header

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Runtime.InteropServices;
using OpenTK.Graphics.OpenGL;

namespace SGLF.Graphics.Vertices
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Structures
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Defines the location, semantics and element types of an specific attribute within a vertex
    /// structure.
    /// </summary>
    public sealed class VertexAttribute
    {
        // ==================================================================================================
        // Properties
        // ==================================================================================================

        /// <summary>
        /// The index of the attribute.
        /// </summary>
        public int AttributeIndex
        {
            get;
            private set;
        }

        /// <summary>
        /// The semantics of the attribute.
        /// </summary>
        public VertexSemantics AttributeSemantics
        {
            get;
            private set;
        }

        /// <summary>
        /// The type of the attribute.
        /// </summary>
        public Type AttributeType
        {
            get;
            private set;
        }

        /// <summary>
        /// The number of components of the attribute.
        /// </summary>
        public int Components
        {
            get;
            private set;
        }

        /// <summary>
        /// The type of each component of the attribute.
        /// </summary>
        public VertexAttribPointerType ComponentsType
        {
            get;
            private set;
        }

        /// <summary>
        /// A boolean indicating whether the attribute should be normalized.
        /// </summary>
        public bool Normalized
        {
            get;
            private set;
        }

        /// <summary>
        /// The offset (in bytes) of the attribute in a vertex.
        /// </summary>
        public int Offset
        {
            get;
            private set;
        }

        /// <summary>
        /// The size (in bytes) of the attribute.
        /// </summary>
        public int SizeInBytes
        {
            get;
            private set;
        }

        /// <summary>
        /// The stride offset (in bytes) from an attribute to the next.
        /// </summary>
        public int Stride
        {
            get;
            private set;
        }

        // ==================================================================================================
        // Constructor
        // ==================================================================================================

        /// <summary>
        /// Initializes a new instance of the <see cref="VertexAttribute"/> class.
        /// </summary>
        /// <param name="attributeIndex">Index of the attribute.</param>
        /// <param name="attributeSemantics">The attribute semantics.</param>
        /// <param name="attributeType">Type of the attribute.</param>
        /// <param name="components">The number of components.</param>
        /// <param name="componentsType">Type of the components.</param>
        /// <param name="normalized">If set to <c>true</c>, the attribute is normalized.</param>
        /// <param name="offset">The offset.</param>
        /// <param name="stride">The stride.</param>
        public VertexAttribute(int attributeIndex, VertexSemantics attributeSemantics,
                               Type attributeType, int components,
                               VertexAttribPointerType componentsType, bool normalized,
                               int offset, int stride)
        {
            AttributeIndex = attributeIndex;
            AttributeSemantics = attributeSemantics;
            AttributeType = attributeType;
            Components = components;
            ComponentsType = componentsType;
            Normalized = normalized;
            Offset = offset;
            Stride = stride;
            SizeInBytes = Marshal.SizeOf(attributeType);
        }
    }
}