﻿/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Dependencies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

using System;
using System.Runtime.InteropServices;
using OpenTK;
using OpenTK.Graphics.OpenGL;

namespace SGLF.Graphics.Vertices
{
    /////////////////////////////////////////////////////////////////////////////////////////////////////////
    // Classes
    /////////////////////////////////////////////////////////////////////////////////////////////////////////

    public struct VertexPNTUC : IVertex
    {
        // ==================================================================================================
        // Static Members
        // ==================================================================================================

        private static readonly VertexFormat format;

        // ==================================================================================================
        // Members
        // ==================================================================================================

        public Vector3 position;
        public Vector3 normal;
        public Vector4 tangent;
        public Vector2 texCoords;
        public Vector4 color;

        // ==================================================================================================
        // Properties
        // ==================================================================================================

        public VertexFormat Format
        {
            get
            {
                return format;
            }
        }

        public Vector3 Position
        {
            get
            {
                return position;
            }
            set
            {
                position = value;
            }
        }

        public Vector3 Normal
        {
            get
            {
                return normal;
            }
            set
            {
                normal = value;
            }
        }

        public Vector4 Tangent
        {
            get
            {
                return tangent;
            }
            set
            {
                tangent = value;
            }
        }

        public Vector2 TexCoords
        {
            get
            {
                return texCoords;
            }
            set
            {
                texCoords = value;
            }
        }

        public Vector4 Color
        {
            get
            {
                return color;
            }
            set
            {
                color = value;
            }
        }

        public VertexPNTUC(Vector3 position, Vector3 normal, Vector4 tangent,
                           Vector2 texCoords, Vector4 color)
        {
            this.position = position;
            this.normal = normal;
            this.tangent = tangent;
            this.texCoords = texCoords;
            this.color = color;
        }

        static VertexPNTUC()
        {
            Type type = typeof(VertexPNTUC);
            int stride = Marshal.SizeOf(type);

            format = new VertexFormat(type,

                new VertexAttribute(0, VertexSemantics.Position, typeof(Vector3), 3,
                                    VertexAttribPointerType.Float, false,
                                    (int) Marshal.OffsetOf(type, "position"), stride),

                new VertexAttribute(1, VertexSemantics.Normal, typeof(Vector3), 3,
                                   VertexAttribPointerType.Float, false,
                                   (int) Marshal.OffsetOf(type, "normal"), stride),

                new VertexAttribute(2, VertexSemantics.Tangent, typeof(Vector4), 4,
                                    VertexAttribPointerType.Float, false,
                                    (int) Marshal.OffsetOf(type, "tangent"), stride),

                new VertexAttribute(3, VertexSemantics.TexCoord, typeof(Vector2), 2,
                                    VertexAttribPointerType.Float, false,
                                    (int) Marshal.OffsetOf(type, "texCoords"), stride),

                new VertexAttribute(4, VertexSemantics.Color, typeof(Vector4), 4,
                                    VertexAttribPointerType.Float, false,
                                    (int) Marshal.OffsetOf(type, "color"), stride)
            );
        }
    }
}