#version 420

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------

#include <Common.inc>

// ----------------------------------------------------------------------------
// Constants
// ----------------------------------------------------------------------------

const int kNumSamples = 32;

const float kFilterRadius = 0.27;

const float kDistanceBias = 0.4;

const float kDistanceThreshold = 0.1;

const float kExponent = 6.5;

const vec2 kPoissonDisk[] = vec2[]
(
    vec2(0.4412978, 0.2829132),
    vec2(0.5798681, 0.7628899),
    vec2(0.4202227, -0.06289911),
    vec2(0.0062838, 0.01322115),
    vec2(0.3175444, 0.6494363),
    vec2(0.9230817, 0.05983341),
    vec2(0.02195309, 0.6010602),
    vec2(0.8156458, 0.5147904),
    vec2(0.1361293, 0.2964344),
    vec2(0.1532432, 0.9791629),
    vec2(0.06008535, -0.336985),
    vec2(0.7438987, -0.2059758),
    vec2(0.4146976, -0.5443072),
    vec2(0.6772702, -0.4732878),
    vec2(-0.1316912, 0.8429648),
    vec2(-0.178405, 0.280784),
    vec2(-0.4493622, 0.6041465),
    vec2(-0.577371, 0.09493348),
    vec2(-0.3086346, -0.1156362),
    vec2(0.5680932, -0.7897843),
    vec2(0.2918109, -0.8102319),
    vec2(-0.6833772, 0.3512088),
    vec2(-0.1375785, -0.7942913),
    vec2(0.0449749, -0.9958646),
    vec2(-0.8869166, 0.01118457),
    vec2(-0.4694583, -0.4168816),
    vec2(-0.72092, -0.2844008),
    vec2(-0.2065955, -0.483092),
    vec2(-0.7221446, 0.6582922),
    vec2(-0.4004281, 0.8847691),
    vec2(-0.4433412, -0.7141169),
    vec2(-0.8312911, -0.5378577)
);

/*
const vec2 kPoissonDisk[] = vec2[]
(
    vec2( -0.94201624,  -0.39906216 ),
    vec2(  0.94558609,  -0.76890725 ),
    vec2( -0.094184101, -0.92938870 ),
    vec2(  0.34495938,   0.29387760 ),
    vec2( -0.91588581,   0.45771432 ),
    vec2( -0.81544232,  -0.87912464 ),
    vec2( -0.38277543,   0.27676845 ),
    vec2(  0.97484398,   0.75648379 ),
    vec2(  0.44323325,  -0.97511554 ),
    vec2(  0.53742981,  -0.47373420 ),
    vec2( -0.26496911,  -0.41893023 ),
    vec2(  0.79197514,   0.19090188 ),
    vec2( -0.24188840,   0.99706507 ),
    vec2( -0.81409955,   0.91437590 ),
    vec2(  0.19984126,   0.78641367 ),
    vec2(  0.14383161,  -0.14100790 )
);
*/
 
// ----------------------------------------------------------------------------
// Attributes
// ----------------------------------------------------------------------------

in vec3 vPosition;
in vec2 vTexCoord;

out vec4 fFragColor;

// ----------------------------------------------------------------------------
// Uniforms
// ----------------------------------------------------------------------------

layout (binding = 0) uniform sampler2D uInputMap;
layout (binding = 1) uniform sampler2D uColorMap1;
layout (binding = 2) uniform sampler2D uColorMap2;
layout (binding = 3) uniform sampler2D uColorMap3;

// ----------------------------------------------------------------------------
// Functions
// ----------------------------------------------------------------------------

vec2 ImageSpaceCoords(vec3 coords)
{
    vec4 projCoord = uScene.Proj * vec4(coords, 1);
    projCoord.xy /= projCoord.w;
    return projCoord.xy * 0.5 + 0.5;
}

void main()
{
    // Obtain current point and normal.
    vec3 point = texture(uColorMap1, vTexCoord).xyz;
    vec3 normal = normalize(texture(uColorMap2, vTexCoord).xyz);
    vec3 viewDir = normalize(-point);
    
    // Compute filter radius in screen space.
    vec2 screenPoint  = ImageSpaceCoords(point);
    vec2 screenRadius = ImageSpaceCoords(point + vec3(kFilterRadius, kFilterRadius, 0));    
    vec2 filterRadius = (screenRadius - screenPoint);
    
    // Approximate AO
    float AO = 1;
    
    for (int i = 0; i < kNumSamples; ++i)
    {
        vec2 texCoord = (kPoissonDisk[i] * filterRadius) + vTexCoord;
        
        vec3 samplePoint = texture(uColorMap1, texCoord).xyz;
        vec3 sampleDir = normalize(samplePoint - point);
 
        float NdotS = max(dot(normal, sampleDir), 0);
        float dist = distance(point, samplePoint);
 
        float a = 1 - smoothstep(0, kDistanceThreshold, dist - kDistanceBias);
        float b = NdotS;
 
        AO += 1 - (a * b);
    }
    
    AO /= kNumSamples;
    AO = pow(AO, kExponent);
    
    fFragColor.rgb = texture(uInputMap, vTexCoord).rgb * AO; 
	//fFragColor.rgb = vec3(AO); 
    fFragColor.a = 1.0;
}
