#version 420

// ----------------------------------------------------------------------------
// Includes
// ----------------------------------------------------------------------------

#include <Common.inc>

// ----------------------------------------------------------------------------
// Attributes
// ----------------------------------------------------------------------------

in vec3 vPosition;
in vec2 vTexCoord;

out vec4 fFragColor;

// ----------------------------------------------------------------------------
// Uniforms
// ----------------------------------------------------------------------------

layout (binding = 1) uniform sampler2D uColorMap1;
layout (binding = 2) uniform sampler2D uColorMap2;
layout (binding = 3) uniform sampler2D uColorMap3;
layout (binding = 4) uniform sampler2D uColorMap4;
layout (binding = 5) uniform sampler2D uDepthMap;

// ----------------------------------------------------------------------------
// Functions
// ----------------------------------------------------------------------------

void main()
{
	vec4 color1 = texture(uColorMap1, vTexCoord);
	vec4 color2 = texture(uColorMap2, vTexCoord);
	vec4 color3 = texture(uColorMap3, vTexCoord);
	vec4 color4 = texture(uColorMap4, vTexCoord);
	
	SurfaceInfo surface = GBufferDecode(color1, color2, color3, color4);

	fFragColor = vec4(0, 0, 0, 1);
	
	#ifdef OPTION_AMBIENT_PASS
    
		fFragColor.rgb += AmbientHemisphere(uAmbientLight, surface);
		fFragColor.rgb *= surface.AmbientOcclusion;
		fFragColor.rgb = mix(fFragColor.rgb, surface.BaseColor, surface.Emission);
		
	#else
		if (surface.Emission == 1)
			discard;
			
		for (int i = 0; i < uNumLights; i++)
			fFragColor.rgb += Light(uLight[i], surface);
	
		fFragColor.rgb *= (1 - surface.Emission);
	#endif
}